#ifndef _Conn_h
#define _Conn_h 1

#include "Conn_config.h"

#include <stdlib.h>
#include <sys/types.h>
#include <fcntl.h>
#include <stdio.h>
#include <sys/socket.h>

struct Conn;
struct Conn_wpool;
struct Conn_split;
struct Conn_ws;

/* Parameters */
enum CONN_PARA {
        CONN_PARA_AUTO_RECONNECT = 0,
	CONN_PARA_RECONNECT_DELAY,
	CONN_PARA_IDLE_TIME,
	CONN_PARA_READ_TIMEOUT,
	CONN_PARA_CONN_TIMEOUT,
	CONN_PARA_TRIGGER,
	CONN_PARA_IBUF,
	CONN_PARA_OBUF
};

/* Callbacks */
enum CONN_CB {
	CONN_CB_ACCEPT = 0,
	CONN_CB_RECV,
	CONN_CB_SEND,
	CONN_CB_DATA,
	CONN_CB_CLOSE,
	CONN_CB_TRIGGER,
	CONN_CB_ERROR,
	CONN_CB_CONNECTED,
	CONN_CB_ACCEPT_ERROR
};

/* misc */
extern void		Log(const unsigned short level, char *format, ...);
extern char		*Conn_dump(const void *buf_src0, const int len_src);
extern char		*Conn_dumphex(const void *buf_src0, const int len_src);
extern void		Conn_debug(int fd, const unsigned short debug);

/* split */
extern void			Conn_split_free(struct Conn_split **s);
extern struct Conn_split	*Conn_split(const char *line0);
extern char			*Conn_split_get_size(const struct Conn_split *s,
					const char *left, unsigned int *size);
extern char			*Conn_split_get_e(const struct Conn_split *s,
					const char *l);
extern char			*Conn_split_get(const struct Conn_split *s,
					const char *l);
extern unsigned long		Conn_split_get_ul(const struct Conn_split *s,
					const char *l, unsigned int base);
extern unsigned long long	Conn_split_get_ull(const struct Conn_split *s,
					const char *l, unsigned int base);
extern double			Conn_split_get_d(const struct Conn_split *s,
					const char *l);

/* conn */
extern unsigned long long	Conn_lifetime(struct Conn *C);
extern char		*Conn_strerror(void);
extern long long	Conn_time_diff(const struct timeval *t1,
				const struct timeval *t2);
extern char		*Conn_status(const unsigned int flags);

extern unsigned int	Conn_iqlen(const struct Conn *C);
extern unsigned int	Conn_oqlen(const struct Conn *C);
extern unsigned int	Conn_qlen(const struct Conn *C);
extern int		Conn_nodelay(const struct Conn *C);
extern void		Conn_rollback(struct Conn *C, const unsigned int bytes);
extern char		*Conn_ibuf(const struct Conn *C);
extern char		*Conn_obuf(const struct Conn *C);
extern unsigned long long	Conn_getid(const struct Conn *C);
extern struct Conn	*Conn_get(const unsigned long long id);
extern void		Conn_last_time(const struct Conn *C, struct timeval *tv);
extern int		Conn_set_cb(struct Conn *C, const unsigned int cb_type,
				void (*f)(struct Conn *));
extern char		*Conn_ostrstr(struct Conn *C, const unsigned int off,
				const char *str, const unsigned int flags);
extern char		*Conn_strstr(struct Conn *C, const char *str);
extern char		*Conn_strcasestr(struct Conn *C, const char *str);
extern char		*Conn_get_line(struct Conn *C);
extern void		Conn_for_every_line(struct Conn *C,
				void (*cb)(struct Conn *C, char *line));
extern int		Conn_printf(struct Conn *C, const char *format, ...);
extern void		Conn_eat(struct Conn *C, const unsigned int bytes);
extern void		Conn_eatall(struct Conn *C);
extern void		Conn_send_done(struct Conn *C);
extern void		Conn_close(struct Conn *C);
extern void		Conn_stop(void);
extern void		Conn_set(struct Conn *C, const unsigned int var,
				const int val);
extern char		*Conn_addr_local(struct Conn *C);
extern char		*Conn_addr_remote(struct Conn *C);
extern int		Conn_port_local(struct Conn *C);
extern int		Conn_port_remote(struct Conn *C);

/* wpool */
extern struct Conn_wpool	*Conn_wpool_create(const unsigned short workers);
extern int			Conn_wpool_destroy(struct Conn_wpool *wp);
extern void			Conn_set_wp(struct Conn *C, struct Conn_wpool *wp);
extern void			Conn_del_wp(struct Conn *C, struct Conn_wpool *wp);

/* conn 2 */
extern int		Conn_init(const unsigned int max);
extern int		Conn_shutdown(void);
extern int		Conn_enqueue_wait(struct Conn *C, const void *buf,
				const size_t count);
extern int		Conn_enqueue(struct Conn *C, const void *buf,
				const size_t count);
void			Conn_kick(struct Conn *C);
extern struct Conn	*Conn_alloc(void);
extern int		Conn_set_socket_domain(struct Conn *C, const int domain);
extern int		Conn_set_socket_type(struct Conn *C, const int type);
extern int		Conn_set_socket_protocol(struct Conn *C, const int proto);
extern int		Conn_set_socket_bind_addr(struct Conn *C, const char *addr);
extern int		Conn_set_socket_addr(struct Conn *C, const char *addr);
extern int		Conn_set_socket_bind_port(struct Conn *C, const int port);
extern int		Conn_set_socket_port(struct Conn *C, const int port);
extern int		Conn_commit(struct Conn *C);
extern struct Conn	*Conn_socket_addr(const int domain, const int type,
				const char *addr, const int port);
extern struct Conn	*Conn_socket(const int domain, const int type,
				const int port);
extern struct Conn	*Conn_connect(const int domain, const int type,
				const char *addr, const int port);
extern int		Conn_band(struct Conn *C, const unsigned int width,
				const unsigned int factor);
extern int		Conn_poll(const int timeout);

/* web server */
void			Conn_ws_free(struct Conn_ws *ws);
int			Conn_ws_create(struct Conn *C);
int			Conn_ws_script(struct Conn *C, const char *url,
				void(*cb)(struct Conn *C));
int			Conn_ws_path(struct Conn *C, const char *url,
				const char *path);


#endif
