/*
 * Test example to see what happends when multiple threads are doing accept
 */

#define _GNU_SOURCE

#include <pthread.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/ip.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/epoll.h>

struct priv
{
	int	master_fd;
};

void *func(void *a)
{
	int r, epollfd;
	struct sockaddr_in peer;
	struct priv *p = a;
	socklen_t size;
	struct epoll_event e;

	epollfd = epoll_create(32);
	memset(&e, 0, sizeof(struct epoll_event));
	e.events = EPOLLIN;

	epoll_ctl(epollfd, EPOLL_CTL_ADD, p->master_fd, &e);

	size = sizeof(struct sockaddr_in);
	while (1) {
		r = epoll_wait(epollfd, &e, 1, -1);
		printf("epoll_wait returned %d\n", r);
		r = accept(p->master_fd, (struct sockaddr *) &peer, &size);
		if (r == -1) {
			perror("accept");
		}
		printf("Accept returned %d\n", r);
	}

	return NULL;
}

int main(void)
{
	pthread_t t[2];
	int i, r, master_fd;
	struct sockaddr_in my;
	struct priv p;

	master_fd = socket(AF_INET, SOCK_STREAM, 0);
	if (master_fd == -1) {
		perror("socket");
		abort();
	}

	memset(&my, 0, sizeof(struct sockaddr_in));
	my.sin_port = htons(3000);
	my.sin_addr.s_addr = INADDR_ANY;

	i = 1;
	setsockopt(master_fd, SOL_SOCKET, SO_REUSEADDR, &i, sizeof(i));

	r = bind(master_fd, (struct sockaddr *) &my, sizeof(struct sockaddr_in));
	if (r !=0) {
		perror("bind");
		abort();
	}

	listen(master_fd, 32);

	p.master_fd = master_fd;

	for (i = 0; i < 2; i++) {
		r = pthread_create(&t[i], NULL, func, &p);
		if (r != 0) {
			perror("pthread_create");
			abort();
		}
	}

	func(&p);

	return 0;
}
