#define _GNU_SOURCE

#include <Conn.h>

/* Global variables */
static unsigned short	debug = 20;

static FILE		*Logf = NULL;
static char		*log_file = "timeout.log";


static void c_connected(struct Conn *C)
{
	Log(4, "%s(A connection was estabilished on slot %d)\n",
		__func__, C->slot);


	Conn_close(C);
}

static void c_close(struct Conn *C)
{
	Log(5, "%s(Slot %d will close)\n",
		__func__, C->slot);
	free(C->private);
}

static void c_data(struct Conn *C)
{
	char *dump;

	if (debug >= 8) {
		dump = Conn_dump(C->ibuf + C->ibuf_head, Conn_qlen(C));
		Log(8, "data: recv: %s\n", dump);
		free(dump);
	}

	Conn_close(C);

	Conn_eatall(C);
}

static void c_error(struct Conn *C)
{
	Log(0, "%s Slot=%u [%s]\n",
		__func__, C->slot, Conn_strerror());
}

int main(void)
{
	char *stat;
	int ret;
	struct Conn *C;

	Logf = fopen(log_file, "a+");
	if (!Logf) {
		fprintf(stderr, "Cannot open log file [%s] [%s]\n",
			log_file, strerror(errno));
		return 1;
	}
	if (debug > 0)
		setlinebuf(Logf);

	Log(0, "Starting...\n");
	Log(0, "\tLogFile=%s Debug=%d\n", log_file, debug);

	Conn_debug(Logf, debug);

	ret = Conn_init(0);
	if (ret == -1) {
		Log(0, "%s", Conn_strerror());
		return 1;
	}

	Conn_connected_cb = c_connected;
	Conn_data_cb = c_data;
	Conn_close_cb = c_close;
	Conn_error_cb = c_error;

	/* Connect to google */
	C = Conn_alloc();
	if (!C) {
		printf("Cannot alloc socket [%s].\n", Conn_strerror());
	} else {
		Conn_set_socket_domain(C, PF_INET);
		Conn_set_socket_type(C, SOCK_STREAM);
		Conn_set_socket_addr(C, "www.google.com");
		Conn_set_socket_port(C, 80);
		ret = Conn_commit(C);
		if (ret != 0) {
			printf("Cannot commit socket [%s].\n",
				Conn_strerror());
		}
	}

	/* Force a timeout for connect */
	Conn_set(C, CONN_PARA_CONN_TIMEOUT, 1);

	while (1) {
		ret = Conn_poll(10);
		if (ret == -1) {
			Log(0, "Error in poll [%s]!\n",
				Conn_strerror());
			break;
		}

		if (ret == 0) {
			Log(0, "Finish work!\n");
			break;
		}

		if (debug >= 9) {
			stat = Conn_status(0);
			Log(9, "%s\n", stat);
			free(stat);
		}
	}

	Conn_shutdown();

	return 0;
}
