#define _GNU_SOURCE

#include <errno.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <time.h>
#include <arpa/inet.h>
#include <stdarg.h>

#include <Conn.h>

/* Global variables */
static unsigned short	debug = 1;


static void trigger(struct Conn *C)
{
	printf("Trigger is running on %llu!\n", Conn_getid(C));
}

int main(void)
{
	char *stat;
	int ret;
	struct Conn *C;

	Conn_debug(NULL, debug);

	ret = Conn_init(0);
	if (ret == -1) {
		printf("%s", Conn_strerror());
		return 1;
	}

	printf("Trying to register IPv4 socket...\n");
	C = Conn_alloc();
	if (!C) {
		printf("Cannot alloc socket [%s].\n", Conn_strerror());
	} else {
		Conn_set_socket_domain(C, PF_INET);
		Conn_set_socket_type(C, SOCK_STREAM);
		Conn_set_socket_bind_addr(C, "127.0.0.1");
		Conn_set_socket_bind_port(C, 60000);
		ret = Conn_commit(C);
		if (ret != 0) {
			printf("Cannot bind on ipv4 socket [%s].\n",
				Conn_strerror());
		}
	}

	/* Install connection specific trigger */
	Conn_set_cb(C, CONN_CB_TRIGGER, trigger);

	/* Force a timeout for connect (in seconds) */
	Conn_set(C, CONN_PARA_TRIGGER, 3);

	while (1) {
		ret = Conn_poll(1000);
		if (ret == -1) {
			printf("Error in poll [%s]!\n",
				Conn_strerror());
			break;
		}

		if (ret == 0) {
			printf("Finish work!\n");
			break;
		}

		if (debug >= 9) {
			stat = Conn_status(0);
			printf("%s\n", stat);
			free(stat);
		}
	}

	Conn_shutdown();

	return 0;
}
