/*
 * Test workers pool
 */

#define _GNU_SOURCE

#include <errno.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <time.h>
#include <arpa/inet.h>
#include <stdarg.h>

#include <Conn.h>

/* Global variables */
static unsigned short	debug = 100;

static void accept_cb(struct Conn *C)
{
	int err;
	char *s = "Hello!\n";

	Log(0, "%s: sending hello!\n", __func__);

	err = Conn_enqueue(C, s, strlen(s));
	if (err == -1) {
		Log(4, "\tCannot enqueue [%s]!\n", Conn_strerror());
		Conn_close(C);
	}
}

static void close_cb(struct Conn *C)
{
	Log(5, "%s: Closing connection with %s/%d id=%llu\n",
		__func__, Conn_addr_remote(C), Conn_port_remote(C),
		Conn_getid(C));
}

static void data_cb_line(struct Conn *C, char *line)
{
	int r;

	if (strncmp(line, "shutdown", 8) == 0)
		exit(1);

	r = Conn_enqueue(C, line, strlen(line));
	if (r == -1)
		Log(0, "Cannot enqueue (%s)!\n", strerror(errno));
}

static void data_cb(struct Conn *C)
{
	Conn_for_every_line(C, data_cb_line);
}

int main(void)
{
	char *stat;
	int ret;
	struct Conn *C;
	struct Conn_wpool *wp;

	Conn_debug(stderr, debug);

	ret = Conn_init(0);
	if (ret == -1) {
		printf("%s", Conn_strerror());
		return 1;
	}

	printf("Trying to register IPv4 socket...\n");
	C = Conn_alloc();
	if (!C) {
		printf("Cannot alloc socket [%s].\n", Conn_strerror());
		return 1;
	}

	Conn_set_socket_domain(C, PF_INET);
	Conn_set_socket_type(C, SOCK_STREAM);
	Conn_set_socket_bind_addr(C, "127.0.0.1");
	Conn_set_socket_bind_port(C, 60000);

	/* Create a pool with 3 workers */
	wp = Conn_wpool_create(3);
	if (wp == NULL) {
		printf("Cannot create wpool!\n");
		return 1;
	}

	Conn_set_wp(C, wp);

	Conn_set_cb(C, CONN_CB_ACCEPT, accept_cb);
	Conn_set_cb(C, CONN_CB_CLOSE, close_cb);
	Conn_set_cb(C, CONN_CB_DATA, data_cb);

	ret = Conn_commit(C);
	if (ret != 0) {
		printf("Cannot bind on ipv4 socket [%s].\n",
			Conn_strerror());
		return 1;
	}

	Log(12, "Waiting...\n");
	while (1) {
		ret = Conn_poll(-1);
		if (ret == -1) {
			printf("Error in poll [%s]!\n",
				Conn_strerror());
			break;
		}

		#if 0
		if (debug >= 9) {
			stat = Conn_status(0);
			printf("%s\n", stat);
			free(stat);
		}
		#endif
	}

	Conn_del_wp(C, wp);

	Conn_shutdown();

	return 0;
}
