/*
 * Test workers pool - web server
 */

#define _GNU_SOURCE

#include <errno.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <time.h>
#include <arpa/inet.h>
#include <stdarg.h>

#include <Conn.h>

/* Global variables */
static unsigned short	debug = 0;

static void data_cb(struct Conn *C)
{
	char body[4096];

	memset(body, 'X', 100);
	body[100] = '\0';

	Conn_printf(C, "HTTP/1.0 200 OK\r\n"
		"Server: wpool2\r\n"
		"Content-Length: 100\r\n"
		"\r\n"
		"%s", body);
	Conn_close(C);
}

int main(void)
{
	int ret;
	struct Conn *C;
	struct Conn_wpool *wp;

	Conn_debug(2, debug);

	ret = Conn_init(0);
	if (ret == -1) {
		printf("%s", Conn_strerror());
		return 1;
	}

	printf("Trying to register IPv4 socket...\n");
	C = Conn_alloc();
	if (!C) {
		printf("Cannot alloc socket [%s].\n", Conn_strerror());
		return 1;
	}

	Conn_set_socket_domain(C, PF_INET);
	Conn_set_socket_type(C, SOCK_STREAM);
	Conn_set_socket_bind_addr(C, "0.0.0.0");
	Conn_set_socket_bind_port(C, 60000);

	/* Create a pool with 2 workers */
	wp = Conn_wpool_create(2);
	if (wp == NULL) {
		printf("Cannot create wpool!\n");
		return 1;
	}

	Conn_set_wp(C, wp);

	Conn_set_cb(C, CONN_CB_DATA, data_cb);
	Conn_set_cb(C, CONN_CB_ACCEPT, NULL);
	Conn_set_cb(C, CONN_CB_CLOSE, NULL);

	ret = Conn_commit(C);
	if (ret != 0) {
		printf("Cannot bind on ipv4 socket [%s].\n",
			Conn_strerror());
		return 1;
	}

	Log(12, "Waiting...\n");
	while (1) {
		ret = Conn_poll(-1);
		if (ret == -1) {
			printf("Error in poll [%s]!\n",
				Conn_strerror());
			break;
		}

		#if 0
		if (debug >= 9) {
			stat = Conn_status(0);
			printf("%s\n", stat);
			free(stat);
		}
		#endif
	}

	Conn_del_wp(C, wp);

	Conn_shutdown();

	return 0;
}
