/*
 * A real web server
 */

#define _GNU_SOURCE

#include <errno.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <time.h>
#include <arpa/inet.h>
#include <stdarg.h>

#include <Conn.h>

/* Global variables */
static unsigned short	debug = 20;

static void data_cb(struct Conn *C)
{
	char body[4096];

	memset(body, 'X', 100);
	body[100] = '\0';

	Conn_printf(C, "HTTP/1.1 200 OK\r\n"
		"Server: wpool2\r\n"
		"Content-Length: 100\r\n"
		"\r\n"
		"%s", body);
	Conn_close(C);
}

void cgi1(struct Conn *C)
{
	char body[4096];

	Log(0, "%s\n", __func__);

	memset(body, 'X', 100);
	body[100] = '\0';

	Conn_printf(C,
		"HTTP/1.1 200 OK\r\n"
		"Server: ws1\r\n"
		"Connection: Keep-Alive\r\n"
		"Content-Length: 100\r\n"
		"Content-Type: text/plain; charset=UTF-8\r\n"
		"\r\n"
		"%s", body);

	Conn_kick(C);
}

int main(void)
{
	int ret;
	struct Conn *C;
	struct Conn_wpool *wp;

	Conn_debug(2, debug);

	ret = Conn_init(0);
	if (ret == -1) {
		printf("%s", Conn_strerror());
		return 1;
	}

	printf("Trying to register IPv4 socket...\n");
	C = Conn_alloc();
	if (!C) {
		printf("Cannot alloc socket [%s].\n", Conn_strerror());
		return 1;
	}

	Conn_set_socket_domain(C, PF_INET);
	Conn_set_socket_type(C, SOCK_STREAM);
	Conn_set_socket_bind_addr(C, "0.0.0.0");
	Conn_set_socket_bind_port(C, 60000);


	/* Create a web server attached to C */
	ret = Conn_ws_create(C);
	if (ret != 0) {
		printf("Cannot create web server!\n");
		return 1;
	}

	ret = Conn_ws_path(C, "/", "/x");
	if (ret != 0) {
		printf("Cannot attach path /x!\n");
		return 1;
	}

	/* When you access http://server/cgi?a=1, you will be redirected to function cgi1 */
	ret = Conn_ws_script(C, "/cgi", cgi1);
	if (ret != 0) {
		printf("Cannot attach script cgi1!\n");
		return 1;
	}


	/* Create a pool with 2 workers */
	wp = Conn_wpool_create(2);
	if (wp == NULL) {
		printf("Cannot create wpool!\n");
		return 1;
	}

	Conn_set_wp(C, wp);

	#if 0
	Conn_set_cb(C, CONN_CB_DATA, data_cb);
	Conn_set_cb(C, CONN_CB_ACCEPT, NULL);
	Conn_set_cb(C, CONN_CB_CLOSE, NULL);
	#endif

	ret = Conn_commit(C);
	if (ret != 0) {
		printf("Cannot bind on ipv4 socket [%s].\n",
			Conn_strerror());
		return 1;
	}

	Log(12, "Waiting...\n");
	while (1) {
		ret = Conn_poll(-1);
		if (ret == -1) {
			printf("Error in poll [%s]!\n",
				Conn_strerror());
			break;
		}

		#if 0
		if (debug >= 9) {
			stat = Conn_status(0);
			printf("%s\n", stat);
			free(stat);
		}
		#endif
	}

	Conn_del_wp(C, wp);

	Conn_shutdown();

	return 0;
}
